#ifndef GLRGUARD

#define GLRGUARD

/**
 *
 * Class <tt>glrGuard</tt> is the base class for classes that represent the nodes of
 * derivation forest (<tt>glrNode</tt> class) and the vertices of graph stack (<tt>glrStateVertex</tt> class).
 * But generally it can be used anywhere. It provides simple semiautomatic garbagge collector.
 * It stores in each object number of pointers to it. If you make a copy of pointer to the
 * <tt>glrGuard</tt> -- type object, you should call it's <tt>shackle()</tt> function which increases
 * the stored number of pointers. If you forgot some pointer to the <tt>glrGuard</tt> -- type object,
 * you should call it's <tt>release()</tt> function before it, which decreases the stored number of pointers and
 * if it become 0, it calls it's own destructor. Therefore objects with the <tt>glrGuard</tt> base class 
 * should be never deleted directly (by the <tt>delete</tt> opearator but by calling their <tt>release()</tt>
 * function. Of course they should never be alocated on stack.
 *
 * Important! <tt>glrGuard</tt> class has virtual destructor, which makes nothing. If you want
 * to get cascade deleting, when one object with <tt>glrGuard</tt> base class stores pointers
 * to other objects with <tt>glrGuard</tt> base class you should simpy call the <tt>release()</tt>
 * function of all remembered object in your own overrided virtual destructor of the first object.
 *
 */
class glrGuard {
  private:
    int numOfPointers;
  public:

    /** 
     *
     * Function <tt>getNumOfPointers()</tt> returns the number of pointers which points to the object.
     * It is here particularly for debug reasons.
     *
     */
    const int &getNumOfPointers() const { return numOfPointers; }

    /**
     *
     * The <tt>release()</tt> function decereases the counter of pointers to this object and
     * deletes object if the counter becomes 0. Call this function before
     * you forgot pointer to an object with base class <tt>glrGuard</tt>.
     * Objects with base class <tt>glrGuard</tt>
     * should be always dealocated using their <tt>release()</tt> function.
     *
     */
    void release() { 
      #ifdef DEBUGEXCEPTIONS
      if(numOfPointers<=0){
        throw glrPointerCounterUnderflowException(numOfPointers);
      }
      #endif
      --numOfPointers;
      if (!numOfPointers) delete this;
    }

    /**
     *
     * Function <tt>shackle()</tt> increases stored counter of pointers to this object.
     * You should call it when you make a copy of the pointer to this object.
     *
     */
    void shackle() { ++numOfPointers; } ;

    /**
     *
     * Constructor initializes the pointers counter to 1 -- you have not to call the <tt>shackle()</tt>
     * after creating a new object with the <tt>glrGuard</tt> base class.
     *
     */
    glrGuard() { numOfPointers=1; } ;

    /**
     *
     * The destructor makes nothing. You should override it in your own <tt>glrGuard</tt> -- derived class
     * and call the <tt>release()</tt> function in it to all remembered <tt>glrGuard</tt> -- derived objects
     * to get the cascading delete if needed. The destructors of the <tt>glrGuard</tt> -- derived classes should be
     * always vitrual.
     *
     */
    virtual ~glrGuard() {
      #ifdef DEBUGEXCEPTIONS
      if (numOfPointers) throw glrInvalidDeleteException(numOfPointers);
      #endif
    };
};

#endif
